﻿using CalendarNotepad.Extends;
using CalendarNotepad.Models;

namespace CalendarNotepad
{
    public partial class Main:Form
    {
        public WorkContext Context { get; set; }
        public Shaking SaveFileShaking { get; set; } = new Shaking(5);

        public Main() {
            this.Context = new WorkContext();

            InitializeComponent();

            //this.Context.LoadFromFile();
            this.Context.GetWorkListFormZipFile();
            var today = DateTime.Now.ToString("yyyyMMdd");
            this.mcWorkDay.TodayDate = DateTime.Now;
            this.rbAllDay.Checked = true;
            var unit = this.Context.GetUnit(today,GetDateRange());
            this.rtbMsg.Text = unit.WorkMessage;
        }

        private void monthCalendar1_DateChanged(object sender,DateRangeEventArgs e) {
            var selectedDate = e.Start.ToString("yyyyMMdd");
            var selectedRange = GetDateRange();
            var unit = this.Context.GetUnit(selectedDate,selectedRange);
            this.rtbMsg.Text = unit.WorkMessage;
        }

        private void monthCalendar1_DateSelected(object sender,DateRangeEventArgs e) {
            //MessageBox.Show("monthCalendar1_DateSelected");
        }

        /// <summary>
        /// 返回当前选择的时间段
        /// </summary>
        /// <returns>时间段</returns>
        private string GetDateRange() =>
            this.rbAllDay.Checked ? "全天" :
            this.rbMorning.Checked ? "上午" :
            this.rbAfternoon.Checked ? "下午" :
            "";

        private void rtbMsg_TextChanged(object sender,EventArgs e) {
            var data = new WorkUnit {
                WorkDay = this.mcWorkDay.SelectionStart.ToString("yyyyMMdd"),
                WorkMessage = this.rtbMsg.Text,
            };
            this.Context.AddOrReplace(data);

            //this.SaveFileShaking.Run(this.Context.SaveToFile);
            this.SaveFileShaking.Run(this.Context.SaveWorkListToZipFile);
        }

        private void Main_FormClosing(object sender,FormClosingEventArgs e) {
            this.SaveFileShaking.RunNow();
        }

        private void Main_Load(object sender,EventArgs e) {
            Task.Factory.StartNew(async () => {
                while(true) {
                    if(this.Context == null) {
                        continue;
                    }
                    var days = new List<DateTime>();
                    var curD = this.mcWorkDay.SelectionStart;
                    var year = curD.Year;
                    var month = curD.Month;
                    for(DateTime i = new DateTime(year,month,1);i.Month == curD.Month;i = i.AddDays(1)) {
                        var df = i.ToString("yyyyMMdd");
                        var m = this.Context.GetUnit(df,GetDateRange());
                        if(m.WorkMessage.IsNotNullOrEmpty()) {
                            days.Add(i);
                        }
                    }
                    this.Invoke(() => {
                        this.mcWorkDay.BoldedDates = days.ToArray();
                    });
                    await Task.Delay(5 * 1000);
                }
            });
        }
    }
}
