﻿using CalendarNotepad.Extends;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.Json;
using System.Text.Json.Nodes;
using System.Threading.Tasks;

namespace CalendarNotepad.Models
{
    /// <summary>
    /// 上下文
    /// </summary>
    public class WorkContext
    {
        /// <summary>
        /// 数据版本
        /// </summary>
        public int Ver { get; set; } = 1;
        /// <summary>
        /// 工作列表
        /// </summary>
        private WorkUnitList WorkList { get; set; } = new WorkUnitList();

        private static object _lockObj = new object();

        public void AddOrReplace(WorkUnit unit) {
            lock(_lockObj) {
                var qu = this.WorkList.Where(a => a.WorkDay == unit.WorkDay);
                if(qu.Any() && unit.WorkMessage.IsNullOrEmpty()) {
                    qu.ToList().ForEach(a => this.WorkList.Remove(a));
                    return;
                }
                if(qu.Any() && unit.WorkMessage.IsNotNullOrEmpty()) {
                    qu.ToList().ForEach(a => a.WorkMessage = unit.WorkMessage);
                    return;
                }
                if(!qu.Any() && unit.WorkMessage.IsNullOrEmpty()) {
                    return;
                }
                if(!qu.Any() && unit.WorkMessage.IsNotNullOrEmpty()) {
                    this.WorkList.Add(unit);
                    return;
                }
            }
        }

        /// <summary>
        /// 获取工作单元
        /// </summary>
        /// <param name="workDay">工作日期</param>
        /// <param name="workRange">工作范围</param>
        /// <returns></returns>
        public WorkUnit GetUnit(string workDay,string workRange) {
            var qu = this.WorkList.Where(a => a.WorkDay == workDay);
            return qu.Any() ? qu.First() : new WorkUnit { WorkDay = workDay,WorkMessage = "" };
        }

        public void SaveToFile() {
            var file = AppConfig.GetSaveFileName;
            if(file.IsNullOrEmpty()) {
                MessageBox.Show("没有配置数据保存文件！");
                return;
            }
            if(!Path.IsPathFullyQualified(file)) {
                file = Path.Combine(Application.StartupPath,file);
            }
            var path = Path.GetDirectoryName(file);
            if(!Directory.Exists(path)) {
                Directory.CreateDirectory(path);
            }
            if(File.Exists(file)) {
                File.Delete(file);
            }
            var jsonModel = new SaveFileModel {
                Ver = this.Ver,WorkList = this.WorkList,
            };
            var str = JsonSerializer.Serialize(jsonModel);
            File.WriteAllText(file,str,Encoding.UTF8);
        }

        public void SaveWorkListToZipFile() {
            var str = JsonSerializer.Serialize(this.WorkList);
            ZipExtend.SaveToFile(str,"WorkContext.json");
        }

        public void GetWorkListFormZipFile() {
            var str = ZipExtend.GetFormFile("WorkContext.json");
            if(str.IsNullOrEmpty()) {
                return;
            }
            this.WorkList = JsonSerializer.Deserialize<WorkUnitList>(str) ?? new WorkUnitList();
        }

        public void LoadFromFile() {
            var file = AppConfig.GetSaveFileName;
            if(file.IsNullOrEmpty()) {
                MessageBox.Show("没有配置数据保存文件！");
                return;
            }
            if(!Path.IsPathFullyQualified(file)) {
                file = Path.Combine(Application.StartupPath,file);
            }
            if(!File.Exists(file)) {
                MessageBox.Show("没有找到数据文件！");
                return;
            }
            try {
                var str = File.ReadAllText(file);
                var obj = JsonSerializer.Deserialize<SaveFileModel>(str);
                if(obj == null) {
                    MessageBox.Show("保存数据格式错误！");
                }
                if(obj?.Ver == 1) {
                    this.WorkList = obj.WorkList;
                }
            }
            catch(Exception ex) {
                MessageBox.Show(ex.ToString());
                return;
            }
        }
    }
}
